<?php 
// FModControl v2.0.0 Copyright (c) Igor Anikeev https://webnav.link/ 2016 - 2024.
?><?php

class FModControl{

private $conf = array();
private $err_count = 0;
private $err_handler = 0;
private $crlf = "\x0D\x0A";
private $new_files_count = 0;
private $modified_files_count = 0;
private $max_launched_time = 0;


public function set_settings($arr){
$this->conf = $arr;
$this->conf['time_limit'] = intval($this->conf['time_limit']);
 if($this->conf['time_limit'] == 0){
 $this->conf['max_launched_time'] = 36000;
 }
 else{
 $this->conf['max_launched_time'] = $this->conf['time_limit'] + 60;
 }
$this->conf['time_diff'] = isset($this->conf['time_diff']) ? intval($this->conf['time_diff']) : 0;
 if(! isset($this->conf['charset'])){
 $this->conf['charset'] = 'utf-8';
 }
 if($this->conf['log_ext'] != '.log' && $this->conf['log_ext'] != '.txt'){
 $this->conf['log_ext'] = '.log';
 }
 foreach($this->conf['check_files'] as $id => $value){
 $this->conf['check_files']["$id"] = str_replace('\\', '/', $value);
 }

 foreach($this->conf['ignore_files'] as $id => $value){
 $this->conf['ignore_files']["$id"] = str_replace('\\', '/', $value);
 }

 if(! isset($this->conf['lang']) || count($this->conf['lang']) < 1){
 $this->conf['lang'] = $this->load_lang('start');
 }
}


public function start(){
 if(! count($this->conf)){
 return "Settings not specified!\n";
 }

 if(function_exists('set_time_limit')){
 @set_time_limit($this->conf['time_limit']);
 }

 if(! is_dir(DATA_DIR)){
 mkdir(DATA_DIR);
 $this->set_mod(DATA_DIR);
 }

 if(! file_exists(DATA_DIR.'/index.html')){
  if(@file_put_contents(DATA_DIR.'/index.html', '') === false){
  return 'Invalid attributes on ' . DATA_DIR . "\n";
  }
 @file_put_contents(DATA_DIR.'/.htaccess', "Deny from all\nRequire all denied\n");
 }

 if(count($this->conf['check_files']) < 1){
 return $this->conf['lang']['Files_to_scan_not_specified'] . "\n";
 }

 if($this->is_script_launched()){
 return $this->conf['lang']['Script_already_running_or_failed_with_error'] . "<br>\n" . $this->conf['lang']['If_previous_launch_failed_next_launch_possible_via']  . ' ' . $this->conf['max_launched_time'] . ' ' . $this->conf['lang']['seconds_after_start_previous'] . "<br>\n" . $this->conf['lang']['To_unlocking_delete_file'] . ' ' . DATA_DIR . '/launched.dat' . ' ' . $this->conf['lang']['not_do_this_if_other_copy_not_completed_work'] . "<br>\n";
 }

 if($this->conf['log_gzip']){
  if(! function_exists('gzopen')){
  $this->conf['log_gzip'] = 0;
  }
 }

$this->set_launched();
$res = $this->check();
$this->delete_old_logs();
$this->set_unlaunched();
return $res;
}


private function check(){
$this->step1_scan();
$this->step2_check();
 if(file_exists(DATA_DIR.'/tree.dat.bak')){
 unlink(DATA_DIR.'/tree.dat.bak');
 }
rename(DATA_DIR.'/tree.dat', DATA_DIR.'/tree.dat.bak');
rename(DATA_DIR.'/tree_tmp.dat', DATA_DIR.'/tree.dat');
file_put_contents(DATA_DIR.'/tree_tmp.dat', '');

$ret = $this->conf['lang']['Completed'] . "<br>\n";

 if($this->conf['info_in_browser']){
 $ret .= $this->conf['lang']['New_files_briefly'] . " $this->new_files_count<br>\n" . $this->conf['lang']['Modified_files_briefly'] . " $this->modified_files_count<br>\n" . $this->conf['lang']['Errors_files_count'] . " $this->err_count<br>\n";
 }

return $ret;
}


private function step1_scan(){
$this->err_handler = fopen(DATA_DIR.'/last_errors.log','wb');
$this->set_mod(DATA_DIR.'/last_errors.log');
$fh=fopen(DATA_DIR.'/tree_tmp.dat','wb');
$this->set_mod(DATA_DIR.'/tree_tmp.dat');
$date=date('Y.m.d H:i:s', time() + $this->conf['time_diff'] * 3600);
fputs($fh, $date . $this->crlf);
 foreach($this->conf['check_files'] as $filename){
  if(! file_exists($filename)){
  $this->put_error($this->conf['lang']['File_or_directory_not_exist'] . ' ' . $filename);
  }
  else{
  $this->scan_dir($filename, $fh);
  }
 }
fclose($fh);
fclose($this->err_handler);
return true;
}


private function step2_check(){

 if(isset($this->conf['no_mtime_compare']) && $this->conf['no_mtime_compare']){
 $fields = array('fname', 'uid', 'gid', 'size', 'md5');
 }
 else{
 $fields = array('fname', 'mode', 'uid', 'gid', 'size', 'mtime', 'ctime', 'md5');
 }

$old_files=array();
$oldfiles_info=array();
 if(! file_exists(DATA_DIR.'/tree.dat')){
 file_put_contents(DATA_DIR.'/tree.dat', '');
 $this->set_mod(DATA_DIR.'/tree.dat');
 }
$fh=fopen(DATA_DIR.'/tree.dat', 'r');
 fgets($fh, 255);
 while(! feof($fh)){
 $str=rtrim(fgets($fh, 2048));
  if($str){
  $sarr=explode('|:|', $str);
  array_push($old_files, $sarr['0']);
  $fileinfo=array();
  $sarr_length=sizeof($sarr);
   for($i=1;$i<$sarr_length;$i++){
    if($sarr["$i"]){
    $fileinfo["$fields[$i]"]=$sarr["$i"];
    }
   }
  array_push($oldfiles_info, $fileinfo);
  }
 }
fclose($fh);

array_multisort($old_files, $oldfiles_info, SORT_STRING);

$oldfiles_size=sizeof($old_files);
$new_files=array();
$modified_files=array();

$fh=fopen(DATA_DIR.'/tree_tmp.dat', 'r');
 fgets($fh, 255);
 while(! feof($fh)){
 $str=rtrim(fgets($fh, 2048));
  if($str){
  $sarr=explode('|:|', $str);
  $fileinfo=array();
  $sarr_length=sizeof($sarr);
   for($i=1;$i<$sarr_length;$i++){
    if($sarr["$i"]){
    $fileinfo["$fields[$i]"]=$sarr["$i"];
    }
   }

   $old_key = $this->get_arr_key($sarr['0'], $old_files, $oldfiles_size);
   if($old_key == -1){
   $logstr = $this->fileinfo_to_str($fileinfo);
   array_push($new_files, "$sarr[0] |$logstr");
   }
   elseif($fileinfo !== $oldfiles_info["$old_key"]){
   $logstr = $this->fileinfo_to_str($fileinfo);
   $logstr_old = $this->fileinfo_to_str($oldfiles_info["$old_key"]);
   array_push($modified_files, "$sarr[0] | New:$logstr Old:$logstr_old");
   }

  }
 }
fclose($fh);

 if(filesize(DATA_DIR.'/last_errors.log') > 5242880){
 $this->conf['res_savings'] = 'mem';
 }

 if($this->conf['res_savings'] == 'mem'){
 unset($old_files, $oldfiles_info);
 }

require_once(INC_DIR.'/mailer.php');
 if(! empty($this->conf['use_smtp'])){
 $mailer = new mailer(array('smtp_conf' => $this->conf['smtp']));
 }
 else{
 $mailer = new mailer();
 }

 if(! sizeof($new_files) && ! sizeof($modified_files) && $this->err_count == 0){
  if($this->conf['mail_no_changes'] && $this->is_admin_email()){
   foreach($this->conf['admin_emails'] as $email){
   $mailer->sendemail('', $email, '', $email, $this->conf['mail_subject'] . ' ' . '(' . $this->conf['lang']['No_changes'] . ')', $this->conf['lang']['No_changes']);
   }
  }
 return true;
 }

$logname = date('Y-m-d_H-i-s', time() + $this->conf['time_diff'] * 3600).$this->conf['log_ext'];
 if($this->conf['log_gzip']){
 $logname.='.gz';
 $log_fh=gzopen(DATA_DIR.'/'.$logname, 'wb');
 }
 else{
 $log_fh=fopen(DATA_DIR.'/'.$logname, 'wb');
 }
$this->set_mod(DATA_DIR.'/'.$logname);

$this->new_files_count = sizeof($new_files);
 if($this->new_files_count){
 $this->put_to_file($log_fh, $this->conf['lang']['NEW_FILES'] . "{$this->crlf}---------{$this->crlf}[NewFiles]{$this->crlf}");
  foreach($new_files as $str){
  $this->put_to_file($log_fh, "$str{$this->crlf}");
  }
 $this->put_to_file($log_fh, "[/NewFiles]{$this->crlf}---------{$this->crlf}" . $this->conf['lang']['New_files_count'] . " {$this->new_files_count}{$this->crlf}{$this->crlf}{$this->crlf}= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = ={$this->crlf}{$this->crlf}{$this->crlf}");
 }

 if($this->conf['res_savings'] == 'mem'){
 unset($new_files);
 }

$this->modified_files_count = sizeof($modified_files);
 if($this->modified_files_count){
 $this->put_to_file($log_fh, $this->conf['lang']['MODIFIED_FILES'] . "{$this->crlf}---------{$this->crlf}[ChangedFiles]{$this->crlf}");
  foreach($modified_files as $str){
  $this->put_to_file($log_fh, "$str{$this->crlf}");
  }
 $this->put_to_file($log_fh, "[/ChangedFiles]{$this->crlf}---------{$this->crlf}" . $this->conf['lang']['Modified_files_count'] . " {$this->modified_files_count}{$this->crlf}{$this->crlf}");
 }

 if($this->conf['res_savings'] == 'mem'){
 unset($modified_files);
 }

$all_cnt = $this->new_files_count + $this->modified_files_count;

 if($all_cnt){
 $this->put_to_file($log_fh, "= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = ={$this->crlf}" . $this->conf['lang']['New_and_modified_files_count'] . " $all_cnt{$this->crlf}");
 }

 if($this->err_count > 0){
 $this->put_to_file($log_fh, "{$this->crlf}= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = ={$this->crlf}" . $this->conf['lang']['Errors_files_count'] . " $this->err_count{$this->crlf}[Errors]{$this->crlf}".file_get_contents(DATA_DIR.'/last_errors.log')."[/Errors]{$this->crlf}");
 }

 if($this->conf['log_gzip']){
 gzclose($log_fh);
 }
 else{
 fclose($log_fh);
 }


 if( ($all_cnt || $this->err_count) && $this->is_admin_email()){
 
 $body = "$logname{$this->crlf}" . $this->conf['lang']['New_files_count'] . " {$this->new_files_count}{$this->crlf}" . $this->conf['lang']['Modified_files_count'] . " {$this->modified_files_count}{$this->crlf}" . $this->conf['lang']['Errors_files_count'] . " $this->err_count";


  if($this->conf['mail_log'] == 'attach'){
  $attach_files = array(DATA_DIR.'/'.$logname);
  }
  else{
  $attach_files = array();
   if($this->conf['mail_log'] == 'body'){
   $body .= $this->crlf . $this->getLogFile(DATA_DIR.'/'.$logname);
   }
  }

   foreach($this->conf['admin_emails'] as $email){
   $mailer->sendemail('', $email, '', $email, $this->conf['mail_subject'], $body, $attach_files);
   }



 }


return true;
}


private function scan_dir($file, $fh){

 if($this->is_ignored_file($file)){
 return true;
 }

$stat = array();
$md5 = '';
 if(is_dir($file)){
  if(! is_link($file)){
  $delimiter = '/';
   if($file == '/' || substr($file, 1) == ':/'){
   $delimiter = '';
   }
  $dh=@opendir($file);
   if(empty($dh)){
   $this->put_error($this->conf['lang']['Failed_to_open_directory'] . ' ' . $file);
   return false;
   }
    while(($fname = readdir($dh)) !== false){
     if($fname != '.' && $fname != '..'){
     $this->scan_dir("$file$delimiter$fname", $fh);
     }
    }
  closedir($dh);
  }
 $stat = @stat($file);
  if(is_array($stat)){
  unset($stat['size'], $stat['mtime'], $stat['ctime']);
  }
  else{
  $this->put_error($this->conf['lang']['Failed_to_get_directory_information'] . ' ' . $file);
  }
 }
 else{
 $stat = @stat($file);
  if(! is_array($stat)){
  $this->put_error($this->conf['lang']['Failed_to_get_file_information'] . ' ' . $file);
  $stat = array('mode' => '', 'uid' => '', 'gid' => '');
  }
  if($this->conf['memorize_hash']){
  $md5 = @md5_file($file);
   if(empty($md5)){
   $this->put_error($this->conf['lang']['Failed_to_open_file'] . ' '.$file);
   }
  }
 }

 if(! isset($stat['size'])){
 $stat['size'] = '';
 }
 if(! isset($stat['mtime'])){
 $stat['mtime'] = '';
 }
 if(! isset($stat['ctime'])){
 $stat['ctime'] = '';
 }

 if(isset($this->conf['no_mtime_compare']) && $this->conf['no_mtime_compare']){
 $finfo = "$file|:|$stat[uid]|:|$stat[gid]|:|$stat[size]|:|$md5";
 }
 else{
 $finfo="$file|:|$stat[mode]|:|$stat[uid]|:|$stat[gid]|:|$stat[size]|:|$stat[mtime]|:|$stat[ctime]|:|$md5";
 }

fputs($fh, $finfo . $this->crlf);
return true;
}



private function tpl_fit($tpl, $file){
$tpl = str_replace('.', '\.', $tpl);
$tpl = str_replace('*', '[^\/]{0,}', $tpl);
$tpl = str_replace('?', '.{1,1}', $tpl);
return preg_match("#^$tpl$#i", $file);
}

private function is_ignored_file($file){
 foreach($this->conf['ignore_files'] as $ignore_tpl){
  if($this->tpl_fit($ignore_tpl, $file)){
  return true;
  }
 }
return false;
}



private function is_script_launched(){
 if(! file_exists(DATA_DIR.'/launched.dat')){
 return false;
 }
$stat = stat(DATA_DIR.'/launched.dat');
 if(time() > ($stat['mtime'] + $this->conf['max_launched_time'])){
 return false;
 }
$data = file_get_contents(DATA_DIR.'/launched.dat');
 if($data == '1'){
 return true;
 }
return false;
}


private function set_launched(){
return file_put_contents(DATA_DIR.'/launched.dat', '1');
}


private function set_unlaunched(){
return file_put_contents(DATA_DIR.'/launched.dat', '');
}


private function get_arr_key($str, $arr, $arr_size){
 if($arr_size < 1){
 return -1;
 }
 elseif($arr_size == 1){
  if(strcmp($str, $arr[0]) == 0){
  return 0;
  }
  else{
  return -1;
  }
 }
$a = 0;
$z = $arr_size;
$defindex = ceil(($z + 0.0) / 2);
$cmp = 0;
 while($defindex != $z){
 $cmp = strcmp($str, $arr[$defindex]);
  if($cmp > 0){
  $a = $defindex;
  $defindex = ceil((($z-$defindex + 0.0) / 2) + $defindex);
  }
  elseif($cmp < 0){
   if($defindex == 1){
	  if(strcmp($str, $arr[0]) == 0){
	  return 0;
	  }
	  else{
	  return -1;
	  }
   }
  $z = $defindex;
  $defindex = ceil((($defindex - $a + 0.0)/2) + $a);
  }
  else{
  return $defindex;
  }
 }
return -1;
}


private function fileinfo_to_str($fileinfo){
 if(! sizeof($fileinfo)){
 return '';
 }
$ret='';
 foreach($fileinfo as $name => $value){
  if($name == 'mode'){
  $value = sprintf('0%o', 0777 & $value);
  }
  elseif($name == 'mtime' || $name == 'ctime'){
  $value = date('Y-m-d H:i:s', $value + $this->conf['time_diff'] * 3600);
  }
 $ret.=" $name=$value;";
 }
return $ret;
}


private function put_to_file($fh, $data){
 if($this->conf['log_gzip']){
 gzwrite($fh, $data);
 }
 else{
 fputs($fh, $data);
 }
}


private function delete_last_slash($text){
 while(substr($text, strlen($text)-1) == '/'){
 $text = substr($text, 0, strlen($text)-1);
 }
return $text;
}

private function check_path($path){
 if($path != '/' && substr($path, 1) != ':/'){
 $path = $this->delete_last_slash($path);
 }
return $path;
}


private function put_error($err){
$this->err_count ++;
$date = date('Y.m.d H:i:s', time() + $this->conf['time_diff'] * 3600);
return fputs($this->err_handler, "$date $err{$this->crlf}");
}


private function set_mod($file){
 if(is_file($file)){
  if($this->conf['files_mod']){
  return @chmod($file, octdec($this->conf['files_mod']));
  }
 }
 elseif(is_dir($file)){
  if($this->conf['dirs_mod']){
  return @chmod($file, octdec($this->conf['dirs_mod']));
  }
 }
}



private function delete_old_logs(){
$this->conf['log_storage_time'] = intval($this->conf['log_storage_time']);
 if($this->conf['log_storage_time'] < 1){
 return true;
 }
$no_actual_time = time() - $this->conf['log_storage_time'] * 86400;
$dh = opendir(DATA_DIR) or die('Cannot open dir: ' . DATA_DIR . "\n");
 while(($fname = readdir($dh)) !== false){
  if($this->is_chk_log_file($fname)){
   if($this->chk_log_date($fname) < $no_actual_time){
   unlink(DATA_DIR.'/'.$fname);
   }
  }
 }
closedir($dh);
}


private function is_chk_log_file($filename){
return preg_match('#^\d\d\d\d-\d\d-\d\d\_\d\d-\d\d-\d\d\.(log|txt){1,1}(\.gz){0,1}$#', $filename);
}


private function chk_log_date($filename){
$filename = preg_replace('#_(\d\d)-(\d\d)-(\d\d)#', ' $1:$2:$3', $filename);
return strtotime(substr($filename, 0, 19));
}


private function is_admin_email(){
 if(! empty($this->conf['admin_emails'][0])){
 return true;
 }
return false;
}


private function getLogFile($file){
 if($this->conf['log_gzip']){
 return $this->gzGetContents($file);
 }
return file_get_contents($file);
}


private function gzGetContents($file){
$data = '';
$fh = gzopen($file, 'rb') or die('Cannot open file ' . $file . "\n");
 while(! gzeof($fh)){
 $data .= gzgets($fh);
 }
gzclose($fh);
return $data;
}



private function load_lang($filename){
$lang = array();
$file = SCRIPT_DIR . "/$filename.lng";
$fh = fopen($file, 'rb') or die('Cannot open ' . $file);
 while(! feof($fh)){
 $lang_str = explode('=', fgets($fh), 2);
 $lang_str[0] = trim($lang_str[0]);
  if($lang_str[0] !== '' && substr($lang_str[0], 0, 1) != '#'){
   if(isset($lang[$lang_str[0]])){
   echo "Dublicate lang $lang_str[0]<br>";
   }
  $lang_str[1] = isset($lang_str[1]) ? trim($lang_str[1]) : '';
   if($lang_str[1] === ''){
   self::err_msg("No value for lang $lang_str[0]");
   }
  $lang[$lang_str[0]] = $lang_str[1];
  }
 }
fclose($fh);
return $lang;
}


}
?>