<?php 
// FModControl v2.0.0 Copyright (c) Igor Anikeev https://webnav.link/ 2016 - 2024.
?><?php

class smtp{

public $display_errors = false;

public $host = 'localhost';
public $port = '25';
public $helo = 'localhost';
public $timeout = 20;
public $auth = false;
public $user = '';
public $pass = '';

public $ssl_certificate_validity_verify = false;
public $ssl_certificate_domain_verify = true;

public $log_errors = false;
public $error_log = 'smtp_err.log';

public $errors = array();
private $handler = '';
private $sock_prefix = '';
private $from = '';
private $to_adrs = '';
private $headers = '';
private $is_connected = false;
private $is_authorized = false;
private $body = '';
private $crlf = '';


public function __construct($params = array()){
 if(defined('DEBUG_MODE') && DEBUG_MODE == 1){
 $this->display_errors = true;
 }
$this->crlf = "\r\n";
$this->is_connected = false;
$this->is_authorized = false;
$this->errors = array();
 if(count($params)){
  foreach($params as $key => $value){
   if(isset($this->$key)){
   $this->$key = $value;
   }
  }
 }
$this->connect();
}


public function connect(){
 if($this->display_errors){
 return $this->connectToHost();
 }
 else{
 return @$this->connectToHost();
 }
}


private function connectToHost(){
 if(465 == $this->port && 'ssl://' !== substr($this->host, 0, 6)){
 $this->sock_prefix = 'ssl://';
 }

$sock_context = stream_context_create(
 array(
 'ssl' => array(
  'verify_peer' => $this->ssl_certificate_validity_verify,
  'verify_peer_name' => $this->ssl_certificate_domain_verify
  )
 )
);


set_error_handler(array($this, 'exceptions_error_handler'));
$err_msg = '';

 try{
 $this->handler = @stream_socket_client($this->sock_prefix . $this->host . ':' . $this->port, $errno, $errstr, $this->timeout, STREAM_CLIENT_CONNECT, $sock_context);
 }
 catch(Exception $e){
 $err_msg = 'SMTP connection error: ' . $e->getMessage() . "<br>\nErrno: $errno<br>\nErrstr: $errstr<br>\n";
 $this->logError($err_msg);
 }

 if($this->display_errors && $err_msg != ''){
 $this->show_error($err_msg);
 }


restore_error_handler();

$welcome = $this->get_data();
 if(is_resource($this->handler)){
 $this->is_connected = true;
  if($this->auth){
  return $this->write_ehlo();
  }
  else{
  return $this->write_helo();
  }
 }
 else{
 $this->add_error("Cannot connect to SMTP server: Errno: $errno; Errstr: $errstr;");
 return false;
 }
}



public function mail_send($send_data){
$this->from = $send_data['from'];
$this->to_adrs = $send_data['to_adrs'];
$this->headers = $send_data['headers'];
$this->body = $send_data['body'];

 if($this->is_connected()){

  if($this->auth && ! $this->is_authorized){
   if(! $this->write_auth()){
   $this->display_err_list();
   return false;
   }
  }

 $this->write_from($this->from);
  if(is_array($this->to_adrs)){
	 foreach($this->to_adrs as $value){
	 $this->write_to($value);
	 }
	}
	else{
	$this->write_to($this->to_adrs);
  }

  if(! $this->write_data()){
  $this->display_err_list();
	return false;
	}

 $headers = str_replace($this->crlf . '.', $this->crlf . '..', trim(implode($this->crlf, $this->headers)));
 $body = str_replace($this->crlf . '.', $this->crlf . '..', $this->body);
 $body = $body[0] == '.' ? '.' . $body : $body;

 $this->write($headers);
 $this->write('');
 $this->write($body);
 $this->write('.');

 $result = (substr(trim($this->get_data()), 0, 3) === '250');
 $this->display_err_list();
 return $result;
 }
 else{
 $this->add_error('Connection error.');
 $this->display_err_list();
 return false;
 }

}




private function write_helo(){
 if(is_resource($this->handler) && $this->write('HELO ' . $this->helo) && substr(trim($error = $this->get_data()), 0, 3) === '250' ){
 return true;
 }
 else{
 $this->add_error('Error HELO command: ' . trim(substr(trim($error),3)));
 return false;
 }
}




private function write_ehlo(){
 if(is_resource($this->handler) && $this->write('EHLO ' . $this->helo) && substr(trim($error = $this->get_data()), 0, 3) === '250'){
 return true;
 }
 else{
 $this->add_error('Error EHLO command: ' . trim(substr(trim($error),3)));
 return false;
 }
}




public function close(){
 if(is_resource($this->handler) && $this->write('QUIT') && substr(trim($error = $this->get_data()), 0, 3) === '221'){
 fclose($this->handler);
 $this->con_status = false;
 return true;
 }
 else{
 $this->add_error('Error QUIT command: ' . trim(substr(trim($error),3)));
 return false;
 }
}




private function write_auth(){
 if(is_resource($this->handler) && $this->write('AUTH LOGIN') && substr(trim($error = $this->get_data()), 0, 3) === '334' && $this->write(base64_encode($this->user)) && substr(trim($error = $this->get_data()),0,3) === '334' && $this->write(base64_encode($this->pass)) && substr(trim($error = $this->get_data()),0,3) === '235'){
 $this->is_authorized = true;
 return true;
 }
 else{
 $this->add_error('Error AUTH command: ' . trim(substr(trim($error),3)));
 return false;
 }
}



private function write_from($from){
 if($this->is_connected() && $this->write('MAIL FROM:<'.$from.'>') && substr(trim($error = $this->get_data()), 0, 2) === '250' ){
 return true;
 }
 else{
 $this->add_error(trim(substr(trim($error), 2)));
 return false;
 }
}



private function write_to($to){
 if($this->is_connected() && $this->write('RCPT TO:<'.$to.'>') && substr(trim($error = $this->get_data()), 0, 2) === '25'){
 return true;
 }
 else{
 $this->add_error(trim(substr(trim($error), 3)));
 return false;
 }
}



private function write_data(){
 if($this->is_connected() && $this->write('DATA') && substr(trim($error = $this->get_data()), 0, 3) === '354' ){
 return true;
 }
 else{
 $this->add_error(trim(substr(trim($error), 3)));
 return false;
 }
}



public function is_connected(){
 if(is_resource($this->handler) && $this->is_connected){
 return true;
 }
return false;
}



private function write($data){
 if(is_resource($this->handler)){
 return fwrite($this->handler, $data . $this->crlf, strlen($data) + 2);
 }
 else{
 return false;
 }
}




private function get_data(){
$ret = '';
$str = '';
$cnt = 0;
 if(is_resource($this->handler)){
  while((strpos($ret, $this->crlf) === false || substr($str,3,1) !== ' ') && $cnt < 100){
  $str = fgets($this->handler, 512);
  $ret .= $str;
  $cnt ++;
  }
 return $ret;
 }
 else{
 return false;
 }
}


private function exceptions_error_handler($severity, $message, $filename, $lineno) {
throw new ErrorException($message, 0, $severity, $filename, $lineno);
}


private function show_error($err){
 if($err != ''){
 echo "<div style=\"border: 1px outset #ff0000;padding:2px;color:#ff0000;\">$err</div>";
 }
}


private function add_error($err){
array_push($this->errors, $err);
}


private function display_err_list(){
$err_msg = '';

 if($this->display_errors && count($this->errors) > 0){
 $err_msg .= "<hr>SMTP Errors list:<br>\n";
 $err_msg .= print_r($this->errors, true);
 $err_msg .= "<hr>\n";
 }

 if($this->display_errors){
 echo $err_msg;
 }

 if($this->log_errors){
 echo $this->logError($err_msg);
 }

}


private function logError($err){
 if($this->log_errors && $err != ''){
  if(ini_get('error_log') == ''){
  @ini_set('error_log', $this->error_log);
  }
 @error_log($err);
 }
}




}
?>